@@ -6,7 +6,7 @@ use warnings;
 
 use Data::Dumper;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =head1 NAME
 
@@ -14,7 +14,7 @@ Protocol::ACME - Interface to the Let's Encrypt ACME API
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =head1 SYNOPSIS
 
@@ -96,30 +96,67 @@ a * are required.
    -----------             --------------------
    *host                   undef
    account_key             undef
-   account_key_path        undef
    openssl                 undef
    ua                      HTTP::Tiny->new()
+   loglevel                error
+   debug                   0
 
 B<host>: The API end point to connect to.  This will generally be acme-staging.api.letsencrypt.org
 or acme-v01.api.letsencrypt.org
 
-B<account_key>: The account private key as a string. DER or PEM format. Excludes C<account_key_path>.
-
-B<account_key_path>: The filesystem path to the account private key. Excludes C<account_key>.
+B<account_key>: The account private key in a scalar ref or filename.  See C<$self->account_key>
+for details on this arguemtn.
 
 B<openssl>: The path to openssl.  If this option is used a local version of the openssl binary will
 be used for crypto operations rather than C<Crypt::OpenSSL::RSA>.
 
 B<ua>: An HTTP::Tiny object customized as you see fit
 
+B<loglevel>: Set the loglevel to one of the C<Log::Any> values.
+
+B<debug>: If set to non-zero this is a shortcut for C<loglevel => debug>
+
 =back
 
 =head2 METHODS
 
 =over
 
+=item account_key( $key_filename )
+
+=item account_key( \$buffer )
+
+=item account_key( \%explicit_args )
+
+
+C<account_key> will load a the private account key if it was not already loaded
+when the C<pProtocol::ACME> object was constructed.  There are three ways to call this:
+
+If the arg is a B<SCALAR> it is assumed to be the filename of the
+key.  C<account_key> will throw an error if there are problems reading the file.
+
+If the arg is a B<SCALAR> reference it is assumed to be a buffer that
+contains the KEY.
+
+If the arg is a B<HASH> reference it contains named arguments.  The arguments
+are:
+
+   KEY          DEFAUL        DESC
+   -----------  -----------   -------------------
+   filename     undef         The key Filename
+   buffer       undef         Buffer containing the key
+   format       undef         Explicitly state the format ( DER | PEM )
+
+If both C<filename> and C<buffer> are set the C<buffer> argument will be ignored.
+
+If the format is not explcitly set C<Protocol::ACME> will look at the key and
+try and determine what the format it.
+
+
 =item load_key_from_disk( $key_path )
 
+B<DEPRECATED>
+
 Load a key from disk.  Currently the key needs to be unencrypted.
 Callbacks for handling password protected keys are still to come.
 
@@ -191,13 +228,38 @@ challenge.
 Called after C<handle_challenge>.  This will poll the challenge status
 resource and will return when the state changes from 'pending'.
 
-=item $cert = sign( $csr )
+=item $cert = sign( $csr_filename )
+
+=item $cert = sign( \$buffer )
+
+=item $cert = sign( \%explicit_args )
 
 
 Call C<sign> after the challenge for each domain ( itentifier ) has
-been fulfilled.  C<$csr> is the DER encoded Certificate Signing
-Request ( CSR ).  On success Let's Encrypt will return the DER encoded
-signed certificate.
+been fulfilled.  There are three ways to call this:
+
+If the arg is a B<SCALAR> it is assumed to be the filename of the
+CSR.  C<sign> will throw an error if there are problems reading the file.
+
+If the arg is a B<SCALAR> reference it is assumed to be a buffer that
+contains the CSR.
+
+If the arg is a B<HASH> reference it contains named arguments.  The arguments
+are:
+
+   KEY          DEFAUL        DESC
+   -----------  -----------   -------------------
+   filename     undef         The CSR Filename
+   buffer       undef         Buffer containing the CSR
+   format       undef         Explicitly state the format ( DER | PEM )
+
+If both C<filename> and C<buffer> are set the C<buffer> argument will be ignored.
+
+If the format is not explcitly set Protocol::ACME will look at the CSR and
+try and determine what the format it.
+
+On success C<Protocol::ACME> will return the DER encoded signed certificate.
+
 
 =item revoke( $certfile )
 
@@ -260,6 +322,17 @@ foreach my $method ( Log::Any::Adapter::Util::detection_methods() ) {
 
 package Protocol::ACME::Exception;
 
+use Data::Dumper;
+
+# very simple stringification ... make this
+# more elaborate according to taste
+use overload ('""' => \&stringify);
+sub stringify
+{
+    my $self = shift;
+    return ref($self).' error: '.Dumper $self;
+}
+
 sub new
 {
   my $class = shift;
@@ -285,7 +358,6 @@ sub new
   return $self;
 }
 
-
 package Protocol::ACME;
 
 use strict;
@@ -301,8 +373,8 @@ use MIME::Base64 qw( encode_base64url decode_base64url decode_base64 encode_base
 use HTTP::Tiny;
 use JSON;
 use Digest::SHA qw( sha256 );
-use Log::Any qw( $log );
-use Log::Any::Adapter ('AcmeLocal', log_level => 'debug' );
+use Log::Any;
+use Log::Any::Adapter;
 
 use Carp;
 
@@ -334,9 +406,20 @@ sub _init
     %$args = @_;
   }
 
-  $self->{host} = $args->{host} if exists $args->{host};
-  $self->{ua} = $args->{ua} if exists $args->{ua};
-  $self->{openssl} = $args->{openssl} if exists $args->{openssl};
+  # TODO: There are more elegant and well baked ways to take care of the
+  #       parameter handling that I am doing here
+  $self->{host}     = $args->{host}    if exists $args->{host};
+  $self->{ua}       = $args->{ua}      if exists $args->{ua};
+  $self->{openssl}  = $args->{openssl} if exists $args->{openssl};
+  $self->{debug}    = $args->{debug}   if exists $args->{debug};
+  $self->{loglevel} = exists $args->{loglevel} ? $args->{loglevel} : "error";
+
+  if ( $self->{debug} )
+  {
+    $self->{loglevel} = "debug";
+  }
+
+  print $self->{loglevel}, "\n";
 
   if ( ! exists $self->{ua} )
   {
@@ -348,40 +431,35 @@ sub _init
     _throw( detail => "host parameter is required for Protocol::ACME::new" );
   }
 
+  $self->{log} = Log::Any->get_logger();
+  Log::Any::Adapter->set('AcmeLocal', log_level => $self->{loglevel} );
+
   if ( exists $args->{account_key} )
   {
-    if (defined $args->{account_key_path}) {
-      _throw( detail => "You cannot submit both account_key and account_key_patho Protocol::ACME::new()." );
-    }
-
-    if ( UNIVERSAL::isa($args->{account_key}, "Crypt::OpenSSL::RSA") )
-    {
-      $self->{key} = $args->{account_key};
-      # TODO: add derivatives
-    }
-    else
-    {
-      $self->load_key( $args->{'account_key'} );
-    }
-  }
-  elsif (exists $args->{account_key_path})
-  {
-    $self->load_key_from_disk( $args->{account_key_path} );
-  }
-  else
-  {
-    _throw( detail => "You must submit one (and only one) of either account_keyr account_key_path." );
+    $self->account_key( $args->{account_key} );
   }
 
   $self->{links}->{directory} = "https://" . $self->{host} . '/directory';
 
   $self->{nonce} = undef;
+
+
 }
 
 sub _throw
 {
   my (@args) = @_;
-  croak Protocol::ACME::Exception->new( { @args } );
+  if ( scalar(@_) == 1 )
+  {
+    @args = ( detail => $_[0] );
+  }
+  croak ( Protocol::ACME::Exception->new( { @args } ) );
+}
+
+sub load_key
+{
+  my ($self, $keystring) = @_;
+  return $self->account_key( \$keystring );
 }
 
 sub load_key_from_disk
@@ -389,27 +467,62 @@ sub load_key_from_disk
   my $self   = shift;
   my $path   = shift;
 
-  my $keystring = _slurp( $path );
-  if ( ! $keystring )
-  {
-    _throw( detail => "Could not open the key file ($path): $!" );
-  }
-
-  return $self->load_key($keystring);
+  return $self->account_key($path);
 }
 
-sub load_key
+sub account_key
 {
-  my ($self, $keystring) = @_;
+  my $self = shift;
+  my $key = shift;
 
-  my $key;
+  my %args = ( filename => undef,
+               buffer   => undef,
+               format   => undef );
+
+  if ( ! ref $key )
+  {
+    $args{filename} = $key;
+  }
+  elsif( ref $key eq "SCALAR" )
+  {
+    $args{buffer} = $$key;
+  }
+  else
+  {
+    map { $args{$_} = $key->{$_} } keys %$key;
+  }
+
+  if ( $args{filename} )
+  {
+    $args{buffer} = _slurp( $args{filename} );
+    if ( ! $args{buffer} )
+    {
+      _throw( "Could not load key from file $args{filename}" );
+    }
+  }
+
+  if ( ! $args{buffer} )
+  {
+    _throw( "Either a buffer or filename must be passed" );
+  }
+
+  if ( ! $args{format} )
+  {
+    $args{format} = Protocol::ACME::Utils::looks_like_pem( $args{buffer} ) ? "PEM" : "DER";
+  }
+
+  my $keystring = $args{buffer};
+  # TODO: This should detect/handle PKCS8-formatted private keys as well.
+  if ( $args{format} eq "DER" )
+  {
+    $keystring = Crypt::Format::der2pem( $keystring, "RSA PRIVATE KEY" );
+  }
 
   if ( exists $self->{openssl} )
   {
     require Protocol::ACME::Key;
-    # TODO: DER format for the openssl path?
     $key = Protocol::ACME::Key->new( keystring => $keystring,
-                                     openssl => $self->{openssl} );
+                                     openssl   => $self->{openssl} );
   }
   else
   {
@@ -425,13 +538,6 @@ sub load_key
       "Protocol::ACME object.  This will use a native openssl binary instead.";
     }
 
-    if ( ! Protocol::ACME::Utils::looks_like_pem($keystring) )
-    {
-      #TODO: This should detect/handle PKCS8-formatted private keys as well.
-      $keystring = Crypt::Format::der2pem( $keystring, "RSA PRIVATE KEY" );
-      print $keystring;
-    }
-
     $key = Crypt::OpenSSL::RSA->new_private_key($keystring);
   }
 
@@ -448,10 +554,13 @@ sub load_key
   $self->{key}->{n} = $n_b64;
   $self->{key}->{e} = $e_b64;
 
-  $log->debug( "Private key loaded" );
+  $self->{log}->debug( "Private key loaded" );
+
 }
 
 
+
+
 sub directory
 {
   my $self = shift;
@@ -470,7 +579,7 @@ sub directory
   @{$self->{links}}{keys %$data} = values %$data;
 
 
-  $log->debug( "Let's Encrypt Directories loaded." );
+  $self->{log}->debug( "Let's Encrypt Directories loaded." );
 }
 
 #
@@ -483,7 +592,7 @@ sub register
   my $msg = encode_json( { resource => 'new-reg' } );
   my $json = $self->_create_jws( $msg );
 
-  $log->debug( "Sending registration message" );
+  $self->{log}->debug( "Sending registration message" );
 
   my $resp = $self->_request_post( $self->{links}->{'new-reg'}, $json );
 
@@ -491,8 +600,8 @@ sub register
   {
     $self->{links}->{'reg'} = $resp->{headers}->{'location'};
 
-    $log->debug( "Known key used" );
-    $log->debug( "Refetching with location URL" );
+    $self->{log}->debug( "Known key used" );
+    $self->{log}->debug( "Refetching with location URL" );
 
     my $json = $self->_create_jws( encode_json( { "resource" => 'reg' } ) );
 
@@ -516,7 +625,7 @@ sub register
     @{$self->{links}}{keys %$links} = values %$links;
 
     $self->{links}->{'reg'} = $resp->{headers}->{'location'};
-    $log->debug( "New key used" );
+    $self->{log}->debug( "New key used" );
   }
   else
   {
@@ -575,11 +684,11 @@ sub accept_tos
 
   if ( exists $self->{reg}->{agreement} )
   {
-    $log->debug( "TOS already accepted. Skipping" );
+    $self->{log}->debug( "TOS already accepted. Skipping" );
     return;
   }
 
-  $log->debug( "Accepting TOS" );
+  $self->{log}->debug( "Accepting TOS" );
   # TODO: check for existance of terms-of-service link
   # TODO: assert on reg url being present
 
@@ -596,7 +705,7 @@ sub accept_tos
 
   if ( $resp->{status} == 202 )
   {
-    $log->debug( "Accepted TOS" );
+    $self->{log}->debug( "Accepted TOS" );
   }
   else
   {
@@ -609,7 +718,7 @@ sub revoke
   my $self = shift;
   my $certfile = shift;
 
-  $log->debug( "Revoking Cert" );
+  $self->{log}->debug( "Revoking Cert" );
 
   my $cert = _slurp( $certfile );
 
@@ -640,7 +749,7 @@ sub authz
   my $self   = shift;
   my $domain = shift;
 
-  $log->debug( "Sending authz message for $domain" );
+  $self->{log}->debug( "Sending authz message for $domain" );
   # TODO: check for 'next' URL and that is it authz
 
   my $msg = _hash_to_json( { "identifier" => { "type" => "dns", "value" => $domain },
@@ -687,7 +796,7 @@ sub handle_challenge
 
   my $fingerprint = encode_base64url( sha256( $jwk ) );
 
-  $log->debug( "Handing challenge for token: $token.$fingerprint" );
+  $self->{log}->debug( "Handing challenge for token: $token.$fingerprint" );
 
   my $ret = $challenge->handle( $token, $fingerprint, @args );
 
@@ -720,10 +829,10 @@ sub check_challenge
   # TODO: check for failure of challenge check
   # TODO: check for other HTTP failures
 
-  $log->debug( "Polling for challenge fulfillment" );
+  $self->{log}->debug( "Polling for challenge fulfillment" );
   while( 1 )
   {
-    $log->debug( "Status: $self->{content}->{status}" );
+    $self->{log}->debug( "Status: $self->{content}->{status}" );
     if ( $self->{content}->{status} eq "pending" )
     {
       sleep(2);
@@ -741,17 +850,45 @@ sub sign
   my $self = shift;
   my $csr = shift;
 
-  $log->debug( "Signing" );
+  $self->{log}->debug( "Signing" );
 
-  # TODO: slurp
-  my $fh = IO::File->new( $csr ) || die $!;
-  my $der;
-  while( <$fh> )
+  my %args = ( filename => undef,
+               buffer   => undef,
+               format   => undef );
+
+  if ( ! ref $csr )
   {
-    $der .= $_;
+    $args{filename} = $csr;
+  }
+  elsif( ref $csr eq "SCALAR" )
+  {
+    $args{Bbuffer} = $$csr;
+  }
+  else
+  {
+    map { $args{$_} = $csr->{$_} } keys %$csr;
   }
-  $fh->close();
 
+  if ( $args{filename} )
+  {
+    $args{buffer} = _slurp( $args{filename} );
+    if ( ! $args{buffer} )
+    {
+      _throw( "Could not load CSR from file $args{filename}" );
+    }
+  }
+
+  if ( ! $args{buffer} )
+  {
+    _throw( "Either a buffer or filename must be passed to sign" );
+  }
+
+  if ( ! $args{format} )
+  {
+    $args{format} = Protocol::ACME::Utils::looks_like_pem( $args{buffer} ) ? "PEM" : "DER";
+  }
+
+  my $der = $args{format} eq "DER" ? $args{buffer} : Crypt::Format::pem2der( $args{buffer} );
 
   my $msg = _hash_to_json( { "resource" => "new-cert", "csr" => encode_base64url( $der ) } );
 
@@ -779,9 +916,6 @@ sub _request_get
 
   my $resp = $self->{ua}->get( $url );
 
-#  print STDERR Dumper( $resp );
-#  print Dumper( $resp );
-
   $self->{nonce} = $resp->{headers}->{$NONCE_HEADER};
   $self->{json} = $resp->{content};
 
